


"""
Module governs tags within a workflow. 
Only the entities in this file should be accessed from external locations. 
"""

from .manager import get
from .manager import register
from .manager import new_group
from .manager import switch_group
