


from galaxy2janis.entities.workflow import Workflow
from galaxy2janis.entities.workflow.step.inputs import (
    InputValue, 
)


def update_components_datatype(value: InputValue, janis: Workflow) -> None:
    pass

