
from . import patterns

from .matches import get_matches
from .matches import get_next_word
from .matches import get_preceeding_dashes
from .matches import get_quoted_sections
from .matches import find_unquoted

from .checks import is_int
from .checks import is_float
from .checks import is_present
from .checks import is_var
from .checks import is_script
from .checks import has_var
